#include <avr/io.h>
#include <stdlib.h>
#include <string.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include "rf01.h"

// send and receive a 16bit-word via spi
uint16_t _rfphy_spi(uint16_t cmd) {
	uint8_t i;
	uint16_t recv = 0;
	// start communication
	RFM12B_SCK_LOW();
	RFM12B_SS_LOW();
	for(i=0; i<16; i++)	{
		if(cmd & 0x8000)
			RFM12B_MOSI_HIGH();
		else
			RFM12B_MOSI_LOW();
		RFM12B_SCK_HIGH();
		_delay_us(1);
		recv<<=1;
		if(RFM12B_MISO_HIGH()) {
			recv|=0x0001;
		}
		RFM12B_SCK_LOW();
		cmd<<=1;
		_delay_us(1);
	}
	RFM12B_SS_HIGH();
	return recv;
}

void setChannel(uint16_t channel) {
	if(channel>3903)
		channel = 3903;
	_rfphy_spi(0xA000|channel);
	//uartWordLn(0xA000|channel);
}

void setBaud(uint8_t baud) {
	_rfphy_spi(0xC800|baud);
	//uartWordLn(0xC800|baud);
}

void rfphyInitHard() {
	// set default values
	RFM12B_SS_HIGH();
	RFM12B_MOSI_HIGH();
	RFM12B_SCK_LOW();
	// init ports
	RFM12B_SCK_INIT();
	RFM12B_MOSI_INIT();
	RFM12B_MISO_INIT();
	RFM12B_SS_INIT();
}

void rfphyInit(void) {

	_rfphy_spi(0x0000);
	_rfphy_spi(0x8000 | MHZ868 | PF125 | KHZ134 | CRYSTAL);
	//uartWordLn(0x8000 | MHZ868 | PF125 | KHZ134 | CRYSTAL);

	setChannel(RF_CHANNEL_0);
	setBaud(RF_BAUD_19K2);

	_rfphy_spi(0xC600 | AFC | FRQ_REG | DEV31 |  OFFS_REG | AUTO_ONLY | OFFS_CALC | STORE_FRQ);
	//uartWordLn(0xC600 | AFC | FRQ_REG | DEV31 |  OFFS_REG | AUTO_ONLY | OFFS_CALC | STORE_FRQ);

	_rfphy_spi(0xC420 |  DIGITAL_FILTER | 2);
	//uartWordLn(0xC420 |  DIGITAL_FILTER | 2);

	_rfphy_spi(0xC200 | CLK166);
	//uartWordLn(0xC200 | CLK166);
	
	_rfphy_spi(0xC000 | VDI_CLOCK );
	//uartWordLn(0xC000 | VDI_CLOCK );

	rfphySetRxMode();
	
	_rfphy_spi(0xC000 | VDI_CLOCK | ENABLE_RECEIVER);
	//uartWordLn(0xC000 | VDI_CLOCK | ENABLE_RECEIVER);
	
}

void rfphySetRxMode() {
	_rfphy_spi(0xCE00 |  FIFOLEVEL(8) | FIFO_RESERVED); // set FIFO mode
	//uartWordLn(0xCE00 | FIFOLEVEL(8) | FIFO_RESERVED);
	_rfphy_spi(0xCE00 | FIFO_RESERVED | FIFOLEVEL(8) | FIFO_ENABLE | FIFO_FILL); // enable FIFO
	//uartWordLn(0xCE00 | FIFO_RESERVED | FIFOLEVEL(8) | FIFO_ENABLE | FIFO_FILL);

}


uint8_t rfphyReceive() {
	uint16_t status = 0;
	uint8_t i;
	RFM12B_SCK_LOW();
	RFM12B_MOSI_LOW();
	RFM12B_SS_LOW();
	for(i=0; i<16; i++)	{
		RFM12B_SCK_HIGH();
		asm volatile ("nop");
		RFM12B_SCK_LOW();
		 asm volatile ("nop");
	}

	uint8_t c = 0;
	for(i=0; i<8; i++)	{
		RFM12B_SCK_HIGH();
		c<<=1;
		if(RFM12B_MISO_HIGH()) {
			c|=0x0001;
		}
		RFM12B_SCK_LOW();
		asm volatile ("nop");
	}
	RFM12B_SS_HIGH();

	if( c == 0x55) {
		rfphySetRxMode();
	}
	return c;
}
